const logger = require('../logger/index');
const pubsubLib = require('@google-cloud/pubsub');

const pubsub = initPubSub();

function wrapInTry(func){
  let result;
  try {
    result = func();
  } catch(e) {
    logger.error(e);
  } finally {
    return result;
  }
}

//incase credentials to pubsub are wrong, we do not want to crash the server
function initPubSub(){
  return wrapInTry( () =>
    pubsubLib(getObjFromBase64(process.env.GOOGLE_CREDENTIALS_B64))
  );
}

function getObjFromBase64(str) {
  const getAscii = () => new Buffer(str, 'base64').toString('ascii');
  return JSON.parse(getAscii(str));
}

function replaceSlashesWithDashes(str) {
  return str.replace(/\//g,'-');
}

function buildPayload(dataObj, optArgs) {
  const { args, req } = dataObj.data;
  const tmp = {
    url: req.route.path,
    args: args,
    session: req.session,
    header: req.headers,
    eventTime: Date.now(),
    results: dataObj.results,
  };

  return (optArgs) ? Object.assign(tmp, optArgs) : tmp;
}

function publish(data, result) {
  const dataObj = {data};
  if(pubsub){
    const topic = pubsub.topic('event');
    const payload = buildPayload(dataObj, result);
    topic.get({ autoCreate: true }, function(err, topic) {
      wrapInTry( () =>
        topic.publish(payload)
      );
    });
  }
};

module.exports = {
  publish,
};

